(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/forms'), require('rxjs')) :
    typeof define === 'function' && define.amd ? define('ngx-datatable-bootstrap4', ['exports', '@angular/core', '@angular/common', '@angular/forms', 'rxjs'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-datatable-bootstrap4'] = {}, global.ng.core, global.ng.common, global.ng.forms, global.rxjs));
}(this, (function (exports, core, common, forms, rxjs) { 'use strict';

    var drag = function (event, _a) {
        var move = _a.move, up = _a.up;
        var pageX = event.pageX, pageY = event.pageY;
        var x = pageX;
        var y = pageY;
        var moved = false;
        var mouseMoveHandler = function (e) {
            // tslint:disable-next-line:no-shadowed-variable
            var pageX = e.pageX, pageY = e.pageY;
            var dx = pageX - x;
            var dy = pageY - y;
            x = pageX;
            y = pageY;
            if (dx || dy) {
                moved = true;
            }
            move(e, dx, dy, x, y);
            e.preventDefault(); // to avoid text selection
        };
        var mouseUpHandler = function (e) {
            // tslint:disable-next-line:no-shadowed-variable
            var pageX = e.pageX, pageY = e.pageY;
            x = pageX;
            y = pageY;
            document.removeEventListener('mousemove', mouseMoveHandler);
            document.removeEventListener('mouseup', mouseUpHandler);
            if (up) {
                up(e, x, y, moved);
            }
        };
        document.addEventListener('mousemove', mouseMoveHandler);
        document.addEventListener('mouseup', mouseUpHandler);
    };

    var defaultTranslations = {
        indexColumn: 'index',
        selectColumn: 'select',
        expandColumn: 'expand',
        paginationLimit: 'Limit',
        paginationRange: 'Results'
    };

    var DataTableColumnDirective = /** @class */ (function () {
        function DataTableColumnDirective() {
            this.ngStyleClass = {};
            this.sortable = false;
            this.resizable = false;
            this.visible = true;
        }
        Object.defineProperty(DataTableColumnDirective.prototype, "cellColor", {
            get: function () {
                if (this.cellColors !== undefined) {
                    return this.cellColors(this);
                }
            },
            enumerable: false,
            configurable: true
        });
        DataTableColumnDirective.prototype.ngOnInit = function () {
            var _a;
            if (!this.styleClass && this.property) {
                if (/^[a-zA-Z0-9_]+$/.test(this.property)) {
                    this.styleClass = 'column-' + this.property;
                }
                else {
                    this.styleClass = 'column-' + this.property.replace(/[^a-zA-Z0-9_]/g, '');
                }
            }
            if (this.styleClass != null) {
                this.ngStyleClass = (_a = {},
                    _a[this.styleClass] = true,
                    _a);
            }
        };
        return DataTableColumnDirective;
    }());
    DataTableColumnDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: 'data-table-column'
                },] }
    ];
    DataTableColumnDirective.propDecorators = {
        header: [{ type: core.Input }],
        sortable: [{ type: core.Input }],
        resizable: [{ type: core.Input }],
        property: [{ type: core.Input }],
        styleClass: [{ type: core.Input }],
        cellColors: [{ type: core.Input }],
        width: [{ type: core.Input }],
        visible: [{ type: core.Input }],
        cellTemplate: [{ type: core.ContentChild, args: ['cellTemplate',] }],
        headerTemplate: [{ type: core.ContentChild, args: ['headerTemplate',] }]
    };

    var DataTableRowComponent = /** @class */ (function () {
        function DataTableRowComponent(dataTable) {
            this.dataTable = dataTable;
            // tslint:disable-next-line:variable-name
            this._selected = false;
            this.expanded = false;
            this.selectedChange = new core.EventEmitter();
            // FIXME is there no template keyword for this in angular 2?
            this._this = this;
        }
        Object.defineProperty(DataTableRowComponent.prototype, "selected", {
            get: function () {
                return this._selected;
            },
            set: function (selected) {
                this._selected = selected;
                this.selectedChange.emit(selected);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableRowComponent.prototype, "displayIndex", {
            // other:
            get: function () {
                if (this.dataTable.pagination) {
                    return this.dataTable.displayParams.offset + this.index + 1;
                }
                else {
                    return this.index + 1;
                }
            },
            enumerable: false,
            configurable: true
        });
        DataTableRowComponent.prototype.getTooltip = function () {
            if (this.dataTable.rowTooltip) {
                return this.dataTable.rowTooltip(this);
            }
            return '';
        };
        DataTableRowComponent.prototype.populateCell = function (column) {
            column.item = this.item;
            column.index = this.index;
            column.selected = this.selected;
            return column;
        };
        DataTableRowComponent.prototype.ngOnDestroy = function () {
            this.selected = false;
        };
        return DataTableRowComponent;
    }());
    DataTableRowComponent.decorators = [
        { type: core.Component, args: [{
                    selector: '[dataTableRow]',
                    template: "<tr class=\"data-table-row\"\n    [title]=\"getTooltip()\"\n    [style.background-color]=\"dataTable.getRowColor(_this)\"\n    [class.row-odd]=\"index % 2 === 0\"\n    [class.row-even]=\"index % 2 === 1\"\n    [class.selected]=\"selected\"\n    [class.clickable]=\"dataTable.selectOnRowClick\"\n    (dblclick)=\"dataTable.rowDoubleClicked(_this, $event)\"\n    (click)=\"dataTable.rowClicked(_this, $event)\">\n  <td *ngIf=\"dataTable.expandColumnVisible\" class=\"row-expand-button\"\n      (click)=\"expanded = !expanded; $event.stopPropagation(); dataTable.rowExpanded(_this, $event)\">\n    <span class=\"fa {{expanded ? 'fa-arrow-circle-o-down' : 'fa-arrow-circle-o-right'}}\"></span>\n  </td>\n  <td *ngIf=\"dataTable.indexColumnVisible\" class=\"index-column\" [textContent]=\"displayIndex\"></td>\n  <td *ngIf=\"dataTable.selectColumnVisible\" class=\"select-column\">\n    <input type=\"checkbox\" [(ngModel)]=\"selected\"/>\n  </td>\n  <ng-container *ngFor=\"let column of dataTable.columns\">\n    <td class=\"data-column\"\n        [ngClass]=\"column.ngStyleClass\"\n        [style.background-color]=\"column.cellColor\"\n        (click)=\"dataTable.cellClicked(populateCell(column), $event)\"\n        *ngIf=\"column.visible\">\n      <div *ngIf=\"!column.cellTemplate\" [textContent]=\"item[column.property]\"></div>\n      <ng-container *ngIf=\"column.cellTemplate\" [ngTemplateOutlet]=\"column.cellTemplate\"\n                    [ngTemplateOutletContext]=\"{column: column, row: _this, item: item}\"></ng-container>\n    </td>\n  </ng-container>\n</tr>\n<tr class=\"row-expansion {{expanded? '' : 'd-none'}}\"\n    *ngIf=\"dataTable.expandableRows && expanded\">\n  <td [attr.colspan]=\"dataTable.columnCount\">\n    <ng-container [ngTemplateOutlet]=\"dataTable.expandTemplate\"\n                  [ngTemplateOutletContext]=\"{row: _this, item: item}\"></ng-container>\n  </td>\n</tr>\n",
                    styles: [".select-column{text-align:center}.row-expand-button{cursor:pointer;text-align:center}.clickable{cursor:pointer}"]
                },] }
    ];
    DataTableRowComponent.ctorParameters = function () { return [
        { type: DataTableComponent, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return DataTableComponent; }),] }] }
    ]; };
    DataTableRowComponent.propDecorators = {
        item: [{ type: core.Input }],
        index: [{ type: core.Input }],
        selectedChange: [{ type: core.Output }]
    };

    var DataTableTitleDirective = /** @class */ (function () {
        function DataTableTitleDirective() {
            this.title = '';
            this.controls = true;
        }
        return DataTableTitleDirective;
    }());
    DataTableTitleDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: 'data-table-title'
                },] }
    ];
    DataTableTitleDirective.propDecorators = {
        title: [{ type: core.Input }],
        titleTemplate: [{ type: core.ContentChild, args: ['titleTemplate',] }],
        controls: [{ type: core.Input }]
    };

    var DataTableComponent = /** @class */ (function () {
        function DataTableComponent() {
            this._sortAsc = true;
            this._offset = 0;
            this._limit = 10;
            this._items = [];
            this._scheduledReload = null;
            this._selectAllCheckbox = false;
            this._reloading = false;
            this._resizeInProgress = false;
            this.subscription = new rxjs.Subscription();
            this.selectedRows = [];
            this.resizeLimit = 30;
            this.pagination = true;
            this.indexColumn = true;
            this.indexColumnHeader = '';
            this.selectColumn = false;
            this.multiSelect = true;
            this.substituteRows = true;
            this.expandableRows = false;
            this.translations = defaultTranslations;
            this.selectOnRowClick = false;
            this.autoReload = true;
            this.showReloading = false;
            // event handlers:
            this.rowClick = new core.EventEmitter();
            this.rowExpand = new core.EventEmitter();
            this.rowDoubleClick = new core.EventEmitter();
            this.headerClick = new core.EventEmitter();
            this.cellClick = new core.EventEmitter();
            this.reload = new core.EventEmitter();
        }
        Object.defineProperty(DataTableComponent.prototype, "items", {
            get: function () {
                return this._items;
            },
            set: function (items) {
                this._items = items;
                this.onReloadFinished();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "sortBy", {
            get: function () {
                return this._sortBy;
            },
            set: function (value) {
                this._sortBy = value;
                this.triggerReload();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "sortAsc", {
            get: function () {
                return this._sortAsc;
            },
            set: function (value) {
                this._sortAsc = value;
                this.triggerReload();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "offset", {
            get: function () {
                return this._offset;
            },
            set: function (value) {
                this._offset = value;
                this.triggerReload();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "limit", {
            get: function () {
                return this._limit;
            },
            set: function (value) {
                this._limit = value;
                this.triggerReload();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "page", {
            // calculated property:
            get: function () {
                return Math.floor(this.offset / this.limit) + 1;
            },
            set: function (value) {
                this.offset = (value - 1) * this.limit;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "lastPage", {
            get: function () {
                return Math.ceil(this.itemCount / this.limit);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "reloading", {
            get: function () {
                return this._reloading;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "displayParams", {
            get: function () {
                return {
                    sortBy: this.sortBy,
                    sortAsc: this.sortAsc,
                    offset: this.offset,
                    limit: this.limit
                };
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "selectAllCheckbox", {
            get: function () {
                return this._selectAllCheckbox;
            },
            set: function (value) {
                this._selectAllCheckbox = value;
                this.onSelectAllChanged(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "columnCount", {
            get: function () {
                var count = 0;
                count += this.indexColumnVisible ? 1 : 0;
                count += this.selectColumnVisible ? 1 : 0;
                count += this.expandColumnVisible ? 1 : 0;
                this.columns.toArray().forEach(function (column) {
                    count += column.visible ? 1 : 0;
                });
                return count;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTableComponent.prototype, "substituteItems", {
            get: function () {
                return Array.from({ length: this.displayParams.limit - this.items.length });
            },
            enumerable: false,
            configurable: true
        });
        DataTableComponent.prototype.getRowColor = function (row) {
            if (this.rowColors !== undefined) {
                return this.rowColors(row);
            }
        };
        // setting multiple observable properties simultaneously
        DataTableComponent.prototype.sort = function (sortBy, asc) {
            this.sortBy = sortBy;
            this.sortAsc = asc;
        };
        DataTableComponent.prototype.reloadItems = function () {
            this._reloading = true;
            this.reload.emit(this._getRemoteParameters());
        };
        DataTableComponent.prototype.rowClicked = function (row, event) {
            this.rowClick.emit({ row: row, event: event });
        };
        DataTableComponent.prototype.rowDoubleClicked = function (row, event) {
            this.rowDoubleClick.emit({ row: row, event: event });
        };
        DataTableComponent.prototype.headerClicked = function (column, event) {
            if (!this._resizeInProgress) {
                this.headerClick.emit({ column: column, event: event });
            }
            else {
                // this is because I can't prevent click from mouseup of the drag end
                this._resizeInProgress = false;
            }
        };
        DataTableComponent.prototype.cellClicked = function (cell, event) {
            this.cellClick.emit({ cell: cell, event: event });
        };
        DataTableComponent.prototype.rowExpanded = function (row, event) {
            this.rowExpand.emit({ row: row, event: event });
        };
        DataTableComponent.prototype.onRowSelectChanged = function (row) {
            // maintain the selectedRow(s) view
            if (this.multiSelect) {
                var index = this.selectedRows.indexOf(row);
                if (row.selected && index < 0) {
                    this.selectedRows.push(row);
                }
                else if (!row.selected && index >= 0) {
                    this.selectedRows.splice(index, 1);
                }
            }
            else {
                if (row.selected) {
                    this.selectedRow = row;
                }
                else if (this.selectedRow === row) {
                    this.selectedRow = undefined;
                }
            }
            // unselect all other rows:
            if (row.selected && !this.multiSelect) {
                this.rows.filter(function (r) { return r.selected; }).forEach(function (r) {
                    if (r !== row) { // avoid endless loop
                        r.selected = false;
                    }
                });
            }
        };
        DataTableComponent.prototype.resizeColumnStart = function (event, column, columnElement) {
            var _this = this;
            this._resizeInProgress = true;
            drag(event, {
                move: function (moveEvent, dx) {
                    if (_this.isResizeInLimit(columnElement, dx)) {
                        column.width = columnElement.offsetWidth + dx;
                    }
                },
            });
        };
        DataTableComponent.prototype.ngOnInit = function () {
            var _this = this;
            this.indexColumnVisible = this.indexColumn;
            this.selectColumnVisible = this.selectColumn;
            this.expandColumnVisible = this.expandableRows;
            this.subscription.add(this.headerClick.subscribe(function (tableEvent) { return _this.sortColumn(tableEvent.column); }));
            if (this.selectOnRowClick) {
                this.subscription.add(this.rowClick.subscribe(function (tableEvent) { return tableEvent.row.selected = !tableEvent.row.selected; }));
            }
            if (this.autoReload && this._scheduledReload == null) {
                this.reloadItems();
            }
        };
        DataTableComponent.prototype.ngOnDestroy = function () {
            this.subscription.unsubscribe();
            this.subscription = undefined;
        };
        DataTableComponent.prototype.onReloadFinished = function () {
            this._selectAllCheckbox = false;
            this._reloading = false;
        };
        DataTableComponent.prototype.triggerReload = function () {
            var _this = this;
            // for avoiding cascading reloads if multiple params are set at once:
            if (this._scheduledReload) {
                clearTimeout(this._scheduledReload);
            }
            this._scheduledReload = setTimeout(function () {
                _this.reloadItems();
            });
        };
        DataTableComponent.prototype._getRemoteParameters = function () {
            var params = {};
            if (this.sortBy) {
                params.sortBy = this.sortBy;
                params.sortAsc = this.sortAsc;
            }
            if (this.pagination) {
                params.offset = this.offset;
                params.limit = this.limit;
            }
            return params;
        };
        DataTableComponent.prototype.sortColumn = function (column) {
            if (column.sortable) {
                var ascending = this.sortBy === column.property ? !this.sortAsc : true;
                this.sort(column.property, ascending);
            }
        };
        DataTableComponent.prototype.onSelectAllChanged = function (value) {
            this.rows.toArray().forEach(function (row) { return row.selected = value; });
        };
        DataTableComponent.prototype.isResizeInLimit = function (columnElement, dx) {
            /* This is needed because CSS min-width didn't work on table-layout: fixed.
             Without the limits, resizing can make the next column disappear completely,
             and even increase the table width. The current implementation suffers from the fact,
             that offsetWidth sometimes contains out-of-date values. */
            return !((dx < 0 && (columnElement.offsetWidth + dx) <= this.resizeLimit) ||
                !columnElement.nextElementSibling || // resizing doesn't make sense for the last visible column
                (dx >= 0 && (columnElement.nextElementSibling.offsetWidth + dx) <= this.resizeLimit));
        };
        return DataTableComponent;
    }());
    DataTableComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'data-table',
                    template: "<data-table-title *ngIf=\"title\"></data-table-title>\n<div class=\"position-relative\">\n  <table class=\"table table-condensed data-table\">\n    <thead>\n    <tr>\n      <th class=\"expand-column-header\" *ngIf=\"expandColumnVisible\">\n      <th class=\"index-column-header\" *ngIf=\"indexColumnVisible\">\n        <span [textContent]=\"indexColumnHeader\"></span>\n      </th>\n      <th class=\"select-column-header\" *ngIf=\"selectColumnVisible\">\n        <input type=\"checkbox\" [(ngModel)]=\"selectAllCheckbox\" *ngIf=\"multiSelect\"/>\n      </th>\n      <ng-container *ngFor=\"let column of columns\">\n        <th class=\"column-header\"\n            #th\n            (click)=\"headerClicked(column, $event)\"\n            [class.sortable]=\"column.sortable\"\n            [class.resizable]=\"column.resizable\"\n            [ngClass]=\"column.ngStyleClass\"\n            [style.width]=\"column.width | px\"\n            *ngIf=\"column.visible\">\n          <span *ngIf=\"!column.headerTemplate\" [textContent]=\"column.header\"></span>\n          <ng-container *ngIf=\"column.headerTemplate\" [ngTemplateOutlet]=\"column.headerTemplate\"\n                        [ngTemplateOutletContext]=\"{column: column}\"></ng-container>\n          <span class=\"column-sort-icon\" *ngIf=\"column.sortable\">\n            <i class=\"fa fa-sort column-sortable-icon\" *ngIf=\"column.property !== sortBy else sortByTemplate\"></i>\n            <ng-template #sortByTemplate>\n              <i class=\"fa {{sortAsc ? 'fa-sort-desc' : 'fa-sort-asc'}}\"></i>\n            </ng-template>\n          </span>\n          <span *ngIf=\"column.resizable\" class=\"column-resize-handle\"\n                (mousedown)=\"resizeColumnStart($event, column, th)\"></span>\n        </th>\n      </ng-container>\n    </tr>\n    </thead>\n    <tbody #row\n           dataTableRow\n           [item]=\"item\"\n           [index]=\"index\"\n           (selectedChange)=\"onRowSelectChanged(row)\"\n           *ngFor=\"let item of items; let index=index\">\n    </tbody>\n    <tbody *ngIf=\"pagination && substituteRows\">\n      <tr\n        [class.row-odd]=\"(index + items.length) % 2 === 0\"\n        [class.row-even]=\"(index + items.length) % 2 === 1\"\n        *ngFor=\"let item of substituteItems, let index = index\">\n        <td *ngIf=\"expandColumnVisible\"></td>\n        <td *ngIf=\"indexColumnVisible\">&nbsp;</td>\n        <td *ngIf=\"selectColumnVisible\"></td>\n        <ng-container *ngFor=\"let column of columns\">\n          <td *ngIf=\"column.visible\">\n        </ng-container>\n      </tr>\n    </tbody>\n  </table>\n  <div class=\"loading-cover\" *ngIf=\"showReloading && reloading\"></div>\n</div>\n<data-table-pagination *ngIf=\"pagination\"></data-table-pagination>\n",
                    encapsulation: core.ViewEncapsulation.None,
                    styles: [".data-table{box-shadow:0 0 15px #ececec;table-layout:fixed}.data-table td{vertical-align:middle}.data-table tbody>tr>td,.data-table thead>tr>th{overflow:hidden}.data-table>tbody+tbody{border-top:none}.data-table .row-odd{background-color:#f6f6f6}.data-table .expand-column-header{width:50px}.data-table .select-column-header{text-align:center;width:50px}.data-table .index-column-header{width:40px}.data-table .column-header{position:relative}.data-table .column-header.sortable{cursor:pointer}.data-table .column-header.resizable{margin-right:8px}.data-table .column-header .column-sort-icon{float:right;margin-right:8px}.data-table .column-header .column-sort-icon .column-sortable-icon{color:#d3d3d3}.data-table .column-header .column-resize-handle{cursor:col-resize;height:100%;margin:0;padding:0;position:absolute;right:0;top:0;width:8px}.loading-cover{background-color:hsla(0,0%,100%,.3);height:100%;position:absolute;top:0;width:100%}"]
                },] }
    ];
    DataTableComponent.propDecorators = {
        itemCount: [{ type: core.Input }],
        pagination: [{ type: core.Input }],
        indexColumn: [{ type: core.Input }],
        indexColumnHeader: [{ type: core.Input }],
        rowColors: [{ type: core.Input }],
        rowTooltip: [{ type: core.Input }],
        selectColumn: [{ type: core.Input }],
        multiSelect: [{ type: core.Input }],
        substituteRows: [{ type: core.Input }],
        expandableRows: [{ type: core.Input }],
        translations: [{ type: core.Input }],
        selectOnRowClick: [{ type: core.Input }],
        autoReload: [{ type: core.Input }],
        showReloading: [{ type: core.Input }],
        rowClick: [{ type: core.Output }],
        rowExpand: [{ type: core.Output }],
        rowDoubleClick: [{ type: core.Output }],
        headerClick: [{ type: core.Output }],
        cellClick: [{ type: core.Output }],
        reload: [{ type: core.Output }],
        title: [{ type: core.ContentChild, args: [core.forwardRef(function () { return DataTableTitleDirective; }),] }],
        columns: [{ type: core.ContentChildren, args: [DataTableColumnDirective,] }],
        rows: [{ type: core.ViewChildren, args: [DataTableRowComponent,] }],
        expandTemplate: [{ type: core.ContentChild, args: ['expandTemplate',] }],
        items: [{ type: core.Input }],
        sortBy: [{ type: core.Input }],
        sortAsc: [{ type: core.Input }],
        offset: [{ type: core.Input }],
        limit: [{ type: core.Input }],
        page: [{ type: core.Input }]
    };

    var DataTablePaginationComponent = /** @class */ (function () {
        function DataTablePaginationComponent(dataTable) {
            this.dataTable = dataTable;
        }
        Object.defineProperty(DataTablePaginationComponent.prototype, "maxPage", {
            get: function () {
                return Math.ceil(this.dataTable.itemCount / this.dataTable.limit);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTablePaginationComponent.prototype, "limit", {
            get: function () {
                return this.dataTable.limit;
            },
            set: function (value) {
                if (Number(value) > 0) {
                    this.dataTable.limit = Math.floor(+value);
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataTablePaginationComponent.prototype, "page", {
            get: function () {
                return this.dataTable.page;
            },
            set: function (value) {
                if (Number(value) > 0) {
                    this.dataTable.page = Math.floor(+value);
                }
            },
            enumerable: false,
            configurable: true
        });
        DataTablePaginationComponent.prototype.pageBack = function () {
            this.dataTable.offset -= Math.min(this.dataTable.limit, this.dataTable.offset);
        };
        DataTablePaginationComponent.prototype.pageForward = function () {
            this.dataTable.offset += this.dataTable.limit;
        };
        DataTablePaginationComponent.prototype.pageFirst = function () {
            this.dataTable.offset = 0;
        };
        DataTablePaginationComponent.prototype.pageLast = function () {
            this.dataTable.offset = (this.maxPage - 1) * this.dataTable.limit;
        };
        DataTablePaginationComponent.prototype.keyPress = function (event) {
            if (!(Number(event.key) >= 0)) {
                event.preventDefault();
            }
        };
        return DataTablePaginationComponent;
    }());
    DataTablePaginationComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'data-table-pagination',
                    template: "<div class=\"navbar p-0 pagination-component\">\n  <div>\n    {{dataTable.translations.paginationRange}}:&nbsp;{{dataTable.offset < 0 ? 0 : dataTable.offset + 1}}&nbsp;-\n    {{[dataTable.offset + dataTable.limit , dataTable.itemCount] | min}}&nbsp;/&nbsp;{{dataTable.itemCount}}\n  </div>\n  <ul class=\"nav justify-content-end\">\n    <li class=\"nav-item pagination-limit\">\n      <div class=\"input-group\">\n        <div class=\"input-group-prepend\">\n          <span class=\"input-group-text\">{{dataTable.translations.paginationLimit}}:</span>\n        </div>\n        <input #limitInput type=\"number\" class=\"form-control\" min=\"1\" step=\"1\"\n               [ngModel]=\"limit\" (blur)=\"limit = limitInput.value\"\n               (keyup.enter)=\"limit = limitInput.value\"\n               (keyup.esc)=\"limitInput.value = limit\"\n               (keypress)=\"keyPress($event)\"/>\n      </div>\n    </li>\n    <li class=\"nav-item\">\n      <button [disabled]=\"dataTable.offset <= 0\" (click)=\"pageFirst()\" class=\"btn btn-default\">&laquo;</button>\n      <button [disabled]=\"dataTable.offset <= 0\" (click)=\"pageBack()\" class=\"btn btn-default\">&lsaquo;</button>\n      <div class=\"d-inline-block\">\n        <div class=\"input-group\">\n          <input #pageInput type=\"number\" class=\"form-control\" min=\"1\" step=\"1\" max=\"{{maxPage}}\"\n                 [ngModel]=\"page\" (blur)=\"page = pageInput.value\"\n                 (keyup.enter)=\"page = pageInput.value\" (keyup.esc)=\"pageInput.value = page\"\n                 (keypress)=\"keyPress($event)\"/>\n          <div class=\"input-group-append\">\n            <span class=\"input-group-text\">/&nbsp;{{dataTable.lastPage}}</span>\n          </div>\n        </div>\n      </div>\n      <button [disabled]=\"(dataTable.offset + dataTable.limit) >= dataTable.itemCount\" (click)=\"pageForward()\"\n              class=\"btn btn-default\">&rsaquo;\n      </button>\n      <button [disabled]=\"(dataTable.offset + dataTable.limit) >= dataTable.itemCount\" (click)=\"pageLast()\"\n              class=\"btn btn-default\">&raquo;\n      </button>\n    </li>\n  </ul>\n</div>\n",
                    styles: [".pagination-component .pagination-limit{margin-right:1rem}.pagination-component .pagination-limit input{width:75px}.pagination-component .btn{margin-top:-3px}"]
                },] }
    ];
    DataTablePaginationComponent.ctorParameters = function () { return [
        { type: DataTableComponent, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return DataTableComponent; }),] }] }
    ]; };

    var DataTableTitleComponent = /** @class */ (function () {
        function DataTableTitleComponent(dataTable) {
            this.dataTable = dataTable;
            this.columnSelectorOpen = false;
        }
        DataTableTitleComponent.prototype.closeSelector = function () {
            this.columnSelectorOpen = false;
        };
        return DataTableTitleComponent;
    }());
    DataTableTitleComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'data-table-title',
                    template: "<div class=\"navbar\">\n  <h4 class=\"title\" [textContent]=\"dataTable.title.title\"></h4>\n  <ng-container [ngTemplateOutlet]=\"dataTable.title.titleTemplate\"\n                [ngTemplateOutletContext]=\"{items: dataTable.items}\"\n                *ngIf=\"dataTable.title.titleTemplate\">\n  </ng-container>\n  <div *ngIf=\"dataTable.title.controls\">\n    <button type=\"button\" class=\"btn btn-default btn-sm refresh-button\"\n            (click)=\"dataTable.reloadItems()\">\n      <i class=\"fa fa-refresh\"></i>\n    </button>\n    <button type=\"button\" class=\"btn btn-default btn-sm column-selector-button\"\n            [class.active]=\"columnSelectorOpen\"\n            (click)=\"columnSelectorOpen = !columnSelectorOpen; $event.stopPropagation()\">\n      <i class=\"fa fa-list\"></i>\n    </button>\n    <div class=\"column-selector-wrapper\" (click)=\"$event.stopPropagation()\">\n      <div *ngIf=\"columnSelectorOpen\" class=\"column-selector-box card\">\n        <div *ngIf=\"dataTable.expandableRows\" class=\"column-selector-fixed-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"dataTable.expandColumnVisible\"/>\n            <span>&nbsp;{{dataTable.translations.expandColumn}}</span>\n        </div>\n        <div *ngIf=\"dataTable.indexColumn\" class=\"column-selector-fixed-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"dataTable.indexColumnVisible\"/>\n            <span>&nbsp;{{dataTable.translations.indexColumn}}</span>\n        </div>\n        <div *ngIf=\"dataTable.selectColumn\" class=\"column-selector-fixed-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"dataTable.selectColumnVisible\"/>\n            <span>&nbsp;{{dataTable.translations.selectColumn}}</span>\n        </div>\n        <div *ngFor=\"let column of dataTable.columns\" class=\"column-selector-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"column.visible\"/>\n            <span>&nbsp;{{column.header}}</span>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n",
                    styles: [".navbar{padding:10px 0}.navbar .title{margin:5px 0 0 5px}.navbar .column-selector-wrapper{position:relative}.navbar .column-selector-wrapper .column-selector-box{box-shadow:0 0 10px #d3d3d3;padding:10px;position:absolute;right:0;top:1px;width:150px;z-index:1060}.navbar .column-selector-wrapper .column-selector-box .checkbox{margin-bottom:4px}.navbar .column-selector-wrapper .column-selector-box .column-selector-fixed-column{font-style:italic}"]
                },] }
    ];
    DataTableTitleComponent.ctorParameters = function () { return [
        { type: DataTableComponent, decorators: [{ type: core.Inject, args: [core.forwardRef(function () { return DataTableComponent; }),] }] }
    ]; };
    DataTableTitleComponent.propDecorators = {
        title: [{ type: core.Input }],
        closeSelector: [{ type: core.HostListener, args: ['document:click',] }]
    };

    var MinPipe = /** @class */ (function () {
        function MinPipe() {
        }
        MinPipe.prototype.transform = function (value) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return Math.min.apply(null, value);
        };
        return MinPipe;
    }());
    MinPipe.decorators = [
        { type: core.Pipe, args: [{
                    name: 'min'
                },] }
    ];

    var PxPipe = /** @class */ (function () {
        function PxPipe() {
        }
        PxPipe.prototype.transform = function (value) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (value) {
                return Number(value) >= 0 ? value + "px" : value;
            }
        };
        return PxPipe;
    }());
    PxPipe.decorators = [
        { type: core.Pipe, args: [{
                    name: 'px'
                },] }
    ];

    var DataTableResource = /** @class */ (function () {
        function DataTableResource(items) {
            this.items = items;
        }
        DataTableResource.prototype.query = function (params, filter) {
            var result = [];
            if (filter) {
                result = this.items.filter(filter);
            }
            else {
                result = this.items.slice(); // shallow copy to use for sorting instead of changing the original
            }
            if (params.sortBy) {
                result.sort(function (a, b) {
                    if (typeof a[params.sortBy] === 'string') {
                        return a[params.sortBy].localeCompare(b[params.sortBy]);
                    }
                    else {
                        return a[params.sortBy] - b[params.sortBy];
                    }
                });
                if (params.sortAsc === false) {
                    result.reverse();
                }
            }
            if (params.offset !== undefined) {
                if (params.limit === undefined) {
                    result = result.slice(params.offset, result.length);
                }
                else {
                    result = result.slice(params.offset, params.offset + params.limit);
                }
            }
            return new Promise(function (resolve, reject) {
                setTimeout(function () { return resolve(result); });
            });
        };
        DataTableResource.prototype.count = function () {
            var _this = this;
            return new Promise(function (resolve, reject) {
                setTimeout(function () { return resolve(_this.items.length); });
            });
        };
        return DataTableResource;
    }());

    var DataTableModule = /** @class */ (function () {
        function DataTableModule() {
        }
        return DataTableModule;
    }());
    DataTableModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        forms.FormsModule
                    ],
                    declarations: [
                        DataTablePaginationComponent,
                        DataTableRowComponent,
                        DataTableComponent,
                        DataTableTitleComponent,
                        DataTableColumnDirective,
                        DataTableTitleDirective,
                        MinPipe,
                        PxPipe,
                    ],
                    exports: [
                        DataTableComponent,
                        DataTableRowComponent,
                        DataTableColumnDirective,
                        DataTableTitleComponent,
                        DataTablePaginationComponent,
                        DataTableTitleDirective
                    ]
                },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.DataTableColumnDirective = DataTableColumnDirective;
    exports.DataTableComponent = DataTableComponent;
    exports.DataTableModule = DataTableModule;
    exports.DataTablePaginationComponent = DataTablePaginationComponent;
    exports.DataTableResource = DataTableResource;
    exports.DataTableRowComponent = DataTableRowComponent;
    exports.DataTableTitleDirective = DataTableTitleDirective;
    exports.defaultTranslations = defaultTranslations;
    exports.ɵa = DataTableTitleComponent;
    exports.ɵb = MinPipe;
    exports.ɵc = PxPipe;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-datatable-bootstrap4.umd.js.map
