import { EventEmitter, OnDestroy, OnInit, QueryList, TemplateRef } from '@angular/core';
import { DataTableCellEvent, DataTableColumn, DataTableHeaderEvent, DataTableParams, DataTableRow, DataTableRowEvent, DataTableTranslations, RowCallback } from '../../models';
import { DataTableColumnDirective } from '../../directives/column.directive';
import { DataTableRowComponent } from '../row/row.component';
import { DataTableTitleDirective } from '../../directives/title.directive';
export declare class DataTableComponent<T> implements OnInit, OnDestroy {
    indexColumnVisible: boolean;
    selectColumnVisible: boolean;
    expandColumnVisible: boolean;
    private _sortBy;
    private _sortAsc;
    private _offset;
    private _limit;
    private _items;
    private _scheduledReload;
    private _selectAllCheckbox;
    private _reloading;
    private _resizeInProgress;
    private subscription;
    selectedRow: DataTableRow<T>;
    selectedRows: DataTableRow<T>[];
    resizeLimit: number;
    itemCount: number;
    pagination: boolean;
    indexColumn: boolean;
    indexColumnHeader: string;
    rowColors: RowCallback<T>;
    rowTooltip: RowCallback<T>;
    selectColumn: boolean;
    multiSelect: boolean;
    substituteRows: boolean;
    expandableRows: boolean;
    translations: DataTableTranslations;
    selectOnRowClick: boolean;
    autoReload: boolean;
    showReloading: boolean;
    rowClick: EventEmitter<DataTableRowEvent<T>>;
    rowExpand: EventEmitter<DataTableRowEvent<T>>;
    rowDoubleClick: EventEmitter<DataTableRowEvent<T>>;
    headerClick: EventEmitter<DataTableHeaderEvent<T>>;
    cellClick: EventEmitter<DataTableCellEvent<T>>;
    reload: EventEmitter<DataTableParams>;
    title: DataTableTitleDirective;
    columns: QueryList<DataTableColumnDirective<T>>;
    rows: QueryList<DataTableRowComponent<T>>;
    expandTemplate: TemplateRef<any>;
    get items(): T[];
    set items(items: T[]);
    get sortBy(): string;
    set sortBy(value: string);
    get sortAsc(): boolean;
    set sortAsc(value: boolean);
    get offset(): number;
    set offset(value: number);
    get limit(): number;
    set limit(value: number);
    get page(): number;
    set page(value: number);
    get lastPage(): number;
    get reloading(): boolean;
    get displayParams(): DataTableParams;
    get selectAllCheckbox(): boolean;
    set selectAllCheckbox(value: boolean);
    get columnCount(): number;
    get substituteItems(): any[];
    getRowColor(row: DataTableRow<T>): string;
    sort(sortBy: string, asc: boolean): void;
    reloadItems(): void;
    rowClicked(row: DataTableRow<T>, event: MouseEvent): void;
    rowDoubleClicked(row: DataTableRow<T>, event: MouseEvent): void;
    headerClicked(column: DataTableColumn<T>, event: MouseEvent): void;
    cellClicked(cell: DataTableColumn<T>, event: MouseEvent): void;
    rowExpanded(row: DataTableRow<T>, event: MouseEvent): void;
    onRowSelectChanged(row: DataTableRow<T>): void;
    resizeColumnStart(event: MouseEvent, column: DataTableColumn<T>, columnElement: HTMLElement): void;
    ngOnInit(): void;
    ngOnDestroy(): void;
    private onReloadFinished;
    private triggerReload;
    private _getRemoteParameters;
    private sortColumn;
    private onSelectAllChanged;
    private isResizeInLimit;
}
